/*
   Copyright (C) 2007 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef AddressLabel_H
#define AddressLabel_H

#include <QLabel>

namespace Mailody
{
/**
 * @class AddressLabel
 * This class is a simple subclass of QLabel, which only adds a signal for
 * right mouse button clicks when hovering over a link.
 * @author Tom Albers <tomalbers@kde.nl>
 */
class AddressLabel : public QLabel
{
    Q_OBJECT

public:

    /**
     * Constructor
     */
    explicit AddressLabel( QWidget* );

    /**
     * Destructor
     */
    ~AddressLabel();

protected:
    /** Reimplemented to catch the mouse click **/
    virtual void mousePressEvent( QMouseEvent* );


// @cond private
private slots:
    void slotLinkHovered( const QString& link );

private:
    QString                     m_link;
// @endcond

signals:
    /**
     * Emitted when a user right clicks on a link.
     * @param link the link under the mouse.
     */
    void rightMouseActivated( const QString &link );

    /**
     * Emittend when a user left clicks on a link.
     * @param link the link under the mouse.
     */
    void leftMouseActivated( const QString &link );
};
}
#endif // AddressLabel_H
