/*
   Copyright (C) 2006-2007 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "messageheaderview.h"
#include "addressbook.h"
#include "addresslabel.h"

// Qt
#include <QApplication>
#include <QClipboard>
#include <QMenu>
#include <QScrollBar>
#include <QTextDocument> // Qt::escape()

// KDE
#include <KDebug>
#include <KConfigGroup>
#include <KFileDialog>
#include <KRun>

#include <kpimutils/linklocator.h>
#include <kio/netaccess.h>

using namespace Mailody;

MessageHeaderView::MessageHeaderView( QWidget* parent )
        : QScrollArea( parent ), m_currentMessage( 0 ),  m_storeAttachments( false )
{
    hide();
    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
    setLineWidth( 1 );
    setFrameStyle( QFrame::Panel | QFrame::Raised );

    m_label = new AddressLabel( widget() );
    m_label->setWordWrap( true );
    setWidget( m_label );

    m_foto = new QLabel( this );

    // linkActivated only seems to work once, use custom implementation.
    connect( m_label, SIGNAL( leftMouseActivated( const QString& ) ),
             SLOT( slotLeftMouseClick( const QString& ) ) );
    connect( m_label, SIGNAL( rightMouseActivated( const QString& ) ),
             SLOT( slotRightMouseClick( const QString& ) ) );
}

MessageHeaderView::~MessageHeaderView()
{}

void MessageHeaderView::setMsg( MessageData* msg )
{
    kDebug() << endl;

    m_currentMessage = msg;
    verticalScrollBar()->setValue( 0 );
    horizontalScrollBar()->setValue( 0 );

    if ( msg && isHidden() ) {
        show(); // can be hidden by favorite website / middle click.
    }

    // keep it after the possible show(), so the first message is
    // shown correctly too.
    updateHeaders();
}

void MessageHeaderView::autoSaveAttachments()
{
    KConfigGroup config = KGlobal::config()->group( "General" );
    m_storeAttachments = config.readEntry( "storeAllAttachments", false );
    m_attachmentFolder = KUrl::fromPath( config.readEntry( "attachmentsFolder" ) );
}

void MessageHeaderView::getFreeFilename( KUrl &fileLocation )
{
    // now we see if a file with that name already exists.
    const QString path = fileLocation.path();
    bool exists = KIO::NetAccess::exists( fileLocation, KIO::NetAccess::DestinationSide, this );
    int i = 0;
    while ( exists ) {
        // it exists, so find the extension, add a digit and try again.
        int extAt = path.lastIndexOf( '.' );
        if ( extAt == -1 )
            extAt = path.length();

        const QString newFilename = path.left( extAt ) + '_' + QString::number( i ) + path.mid( extAt );
        fileLocation.setPath( newFilename );

        exists = KIO::NetAccess::exists( fileLocation, KIO::NetAccess::DestinationSide, this );
        i++;
    }
}

void MessageHeaderView::autoSaveAttachment( const KUrl &fileLocation, const QString &fileName )
{
    KUrl destination( m_attachmentFolder );
    destination.addPath( fileName );
    getFreeFilename( destination );

    kDebug() << "Filelocation" << fileLocation << "Visible Name" << fileName << "Destination" << destination;

    KIO::NetAccess::file_copy( fileLocation, destination, this );
}

void MessageHeaderView::updateHeaders()
{
    if ( !m_currentMessage )
        return;

    bool hasPhoto=false;
    /* TODO: PORT
       // copied from KMail...
    QString photoURL;
    int photoWidth = 60;
    int photoHeight = 60;

    KABC::AddresseeList addresses =
        Addressbook::instance()->findByEmail(m_currentMessage->sender_email());
    if (addresses.count()==1)
    {

        // Next part is an example of how the on hover show more info.
        // as it needs to do the same for all addresses in the headers
        // I need to think about this more (it will require an address
        // book search for every address for example. Remove next line when
        // and uncomment to see it.

        QString tip;
        PhoneNumber::List list = addresses[0].phoneNumbers();
        QValueList<PhoneNumber>::Iterator it=list.begin();
        for(; it!=list.end(); ++it)
        {
            tip.append((*it).label() + ": " + (*it).number() );
            tip.append("\n");
            kDebug() << tip << endl;
        }

        if (!tip.isEmpty())
        {
            m_sender_data->setUseTips(true);
            m_sender_data->setTipText(tip);
        }
        else
            m_sender_data->setUseTips(false);

        // stop here!

        if (addresses[0].photo().isIntern())
        {
            QImage photo = addresses[0].photo().data();
            if ( !photo.isNull() )
            {
                photoWidth = photo.width();
                photoHeight = photo.height();
                // scale below 60, otherwise it can get way too large
                if ( photoHeight > 60 ) {
                    double ratio = ( double )photoHeight / ( double )photoWidth;
                    photoHeight = 60;
                    photoWidth = (int)( 60 / ratio );
                    photo = photo.scaled( photoWidth, photoHeight );
                }

                Q3MimeSourceFactory::defaultFactory()->
                        setImage( "UID"+addresses[0].uid(), photo );

                m_foto->setText(
                        QString("<img src=\"%1\" width=\"%2\" height=\"%3\">")
                                .arg( "UID"+addresses[0].uid() )
                                .arg( photoWidth )
                                .arg( photoHeight ));
                hasPhoto = true;
            }
        }
    }
    else
        m_foto->setText(QString::null);
    // --- end

    */
    //------------- the rest
    using KPIMUtils::LinkLocator;
    const int flags = LinkLocator::PreserveSpaces |
                      LinkLocator::HighlightText |
                      LinkLocator::IgnoreUrls;

    QString text = "<table style=\"margin-right: 80px; margin-left:4px;\" ";
    text.append( "cellpadding=0 cellspacing=1 width=\""
                 + QString::number( width() ) + "\">" );
    text.append( "<tr><td colspan=2>"
                 + LinkLocator::convertToHtml( m_currentMessage->subject()
                                               .trimmed(), flags )
                 + "</td></tr>" );

    QString t = "<a href=\"email:"
                + LinkLocator::convertToHtml(
                    m_currentMessage->sender_full( false ),
                    flags ) + "\">"
                + LinkLocator::convertToHtml(
                    m_currentMessage->sender_full( true )
                    .trimmed(), flags ) + "</a>";
    text.append( "<tr><td valign=top align=right>" + i18n( "From" ) + ": </td>" );
    if ( m_currentMessage->userAgent().startsWith( QLatin1String( "Mailody" ) ) )
        text.append( "<td valign=top width=\"100%\"><font color=\"#22cc22\">"
                     + t + "<td></tr>\n" );
    else
        text.append( "<td valign=top width=\"100%\">" + t + "</td></tr>" );

    QStringList to = m_currentMessage->to_list();
    QString toList;
    if ( to.count()>0 ) {
        QStringList::Iterator it=to.begin();
        for ( ; it!=to.end(); ++it ) {
            if ( !toList.isEmpty() )
                toList.append( ", " );
            toList.append( "<a href=\"email:"
                           + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                           + "\">"
                           + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                           + "</a>" );
        }
        text.append( "<tr><td align=right valign=top>" + i18n( "To" ) + ": </td>" );
        text.append( "<td valign=top width=\"100%\">" + toList + "</td></tr>" );
    }

    QStringList cc = m_currentMessage->cc_list();
    if ( cc.count()>0 ) {
        QString t;
        QStringList::Iterator it=cc.begin();
        for ( ; it!=cc.end(); ++it ) {
            if ( !t.isEmpty() )
                t.append( ", " );
            t.append( "<a href=\"email:"
                      + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                      + "\">"
                      + LinkLocator::convertToHtml(( *it ).trimmed(), flags )
                      + "</a>" );

        }
        text.append( "<tr><td valign=top align=right>"
                     + i18n( "Cc" )+": </td>" );
        text.append( "<td valign=top width=\"100%\">" + t + "</td></tr>" );
    }

    QString resentFrom = m_currentMessage->resentFrom().trimmed();
    if ( !resentFrom.isEmpty() ) {
        QString t = "<a href=\"email:"
                    + LinkLocator::convertToHtml( resentFrom, flags ) +"\">"
                    + LinkLocator::convertToHtml( resentFrom, flags ) + "</a>";

        QString t2;
        QString resentTo = m_currentMessage->resentTo().trimmed();
        if ( resentTo.isEmpty() )
            t2 = toList;
        else
            t2 = "<a href=\"email:"
                 + LinkLocator::convertToHtml( resentTo, flags ) +"\">"
                 + LinkLocator::convertToHtml( resentTo, flags ) + "</a>";

        text.append( "<tr><td valign=top align=right><nobr>" +
                     i18n( "Resent" )+": </nobr></td>" );
        text.append( "<td valign=top>"
                     + i18nc( "resent by from-address(1) to to-address(2)",
                              "by %1 to %2", t, t2 ) + "</td></tr>" );
    }

    QHash<KUrl,QString> list = m_currentMessage->attachments();
    QHash<KUrl,QString>::Iterator ita;
    if ( !list.isEmpty() ) {
        QString t;
        for ( ita = list.begin(); ita != list.end(); ++ita ) {
            if ( !t.isEmpty() )
                t.append( ", " );
            t.append( "<a href=\"attachment:"+ita.key().path()+"\">" +
                      Qt::escape( ita.value().trimmed() ) + "</a>" );

            if ( m_storeAttachments )
                autoSaveAttachment( ita.key(), ita.value() );
        }
        text.append( "<tr><td align=right valign=top>"
                     + i18n( "Attachments" )+": </td>" );
        text.append( "<td valign=top align=left>" + t + "</td></tr>" );
    }

    text.append( "</table>" );

    m_label->setText( text );
    m_label->adjustSize();

    if ( m_label->height()+6 < 100 )
        setMaximumHeight( m_label->height()+6 );
    else
        setMaximumHeight( 100 );

    if ( !hasPhoto )
        m_label->setFixedWidth( viewport()->width()-5 );
    else
        m_label->setFixedWidth( viewport()->width()-60 );
}

void MessageHeaderView::clearView()
{
    m_label->setText( "<table width=\"100%\" height=\"100%\">"
                      "<tr height=\"100%\"><td width=\"100%\">&nbsp;</td></tr></table>" );
    m_foto->setText( QString() );
    hide();
}

void MessageHeaderView::slotLeftMouseClick( const QString& to )
{
    kDebug();
    if ( !m_currentMessage )
        return;

    if ( to.startsWith( QLatin1String( "email:" ) ) ) {
        Akonadi::Collection box;
        if ( m_currentMessage )
            box = m_currentMessage->collection();

        emit openComposer( box, to.mid( 6 ) );
    } else if ( to.startsWith( QLatin1String( "attachment:" ) ) ) {
        QString attachment = to.mid( 11 );
        KRun *run = new KRun( attachment, this );
        run->setRunExecutables( false );
    }
}

void MessageHeaderView::slotRightMouseClick( const QString& to )
{
    kDebug() << to << endl;
    if ( to.startsWith( QLatin1String( "email:" ) ) ) {
        QString address = to.mid( 6 );

        QMenu* p = new QMenu( this );
        QAction* newmsg = new QAction( i18n( "New Message to..." ), this );
        newmsg->setIcon( KIcon( "mail-message-new" ) );
        QAction* add = new QAction( i18n( "Add to KAddressBook..." ), this );
        add->setIcon( KIcon( "contact-new" ) );
        QAction* copy = new QAction( i18n( "Copy" ), this );
        copy->setIcon( KIcon( "edit-copy" ) );

        p->addAction( newmsg );
        p->addAction( add );
        p->addSeparator();
        p->addAction( copy );

        QAction* choice = p->exec( QCursor::pos() );

        if ( choice == add )
            Addressbook::instance()->add( address );
        else if ( choice == newmsg )
            slotLeftMouseClick( to );
        else if ( choice == copy ) {
            QClipboard *cb = QApplication::clipboard();
            cb->setText( address, QClipboard::Clipboard );
        }
    } else if ( to.startsWith( QLatin1String( "attachment:" ) ) ) {
        QString attachment = to.mid( 11 );

        QMenu* p = new QMenu( this );
        QAction* open = new QAction( i18n( "Open" ), this );
        QAction* openwith = new QAction( i18n( "Open with..." ), this );
        QAction* save = new QAction( i18n( "Save..." ), this );
        QAction* saveAll = new QAction( i18n( "Save All..." ), this );

        p->addAction( open );
        p->addAction( openwith );
        p->addAction( save );
        p->addAction( saveAll );

        QAction* choice = p->exec( QCursor::pos() );

        KUrl url( attachment );
        if ( choice == open ) {
            KRun *run = new KRun( url, this );
            run->setRunExecutables( false );
        } else if ( choice == openwith )
            KRun::displayOpenWithDialog( KUrl::List::List( url ), this );
        else if ( choice == save )
            slotLeftMouseClick( to );
        else if ( choice == saveAll ) {
            KUrl base = KFileDialog::getExistingDirectoryUrl(
                            KUrl( "kfiledialog:///saveAttachmentsFolder" ), this );
            if ( !base.hasPath() )
                return;

            QHash<KUrl,QString> aMap = m_currentMessage->attachments();
            QHash<KUrl,QString>::Iterator ita;
            for ( ita = aMap.begin(); ita != aMap.end(); ++ita ) {
                KUrl destination( base );
                destination.addPath( ita.value() );
                getFreeFilename( destination );

                kDebug() << "copy: " << ita.key() << "->" << destination;

                KIO::NetAccess::file_copy( ita.key(), destination, this );
            }
        }
    }
}

#include "messageheaderview.moc"
