/*
   Copyright (C) 2008 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef MESSAGEROUTEWIDGET_H
#define MESSAGEROUTEWIDGET_H

#include <KDialog>
#include <kio/job.h>
#include "ip2geohostip.h"

#include <QDomElement>
#include <QHash>

class KProgressDialog;

namespace Marble
{
class MarbleWidget;
}

namespace Mailody
{

class MessageData;

/**
 * @class MessageRouteWidget
 * This class is a widget which shows the geo route a message has taken.
 * It determines the ip-addressess from the message, retrieves the
 * coordinates, and plots that on a marble widget.
 *
 * chunks of code are copied from Digikam.
 *
 * @author Tom Albers <tomalbers@kde.nl>
 */
class MessageRouteWidget : public KDialog
{
    Q_OBJECT

public:
    /**
      * Constructor
      * @param parent the parent widget
      */
    explicit MessageRouteWidget( QWidget* parent );

    /**
      * Set the ip-addresses
      */
    void setIPs( const QStringList& );

protected:
    void accept();

// @cond private
private slots:
    void slotReply1( GeoInfo info );
    void slotReply2( GeoInfo info );

private:
    void getIPMethod1( const QString& ip );
    void getIPMethod2( const QString& ip );
    void addIPsToWidget();
    QDomElement addKmlElement( QDomDocument &kmlDocument, QDomElement &target, const QString& tag );
    QDomElement addKmlTextElement( QDomDocument &kmlDocument, QDomElement &target,
                                   const QString& tag, const QString& text );

    MessageData *m_message;
    KProgressDialog *m_progress;
    Marble::MarbleWidget *m_marble;
    ip2geoHostip* m_ip2geoHostip;
    QStringList m_ips;
    QHash<QString, GeoInfo> m_ipinfo;
// @endcond
};

}

#endif
