/*
   Copyright (C) 2006-2007 Omat Holding B.V. <info@omat.nl>
   Copyright (C) 2007 Frode M. Døving <frode@lnix.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

// Own
#include "messageview.h"
#include "messagebodyview.h"
#include "messageheaderview.h"
#include "nepomukwidget.h"

// Qt
#include <QPrintPreviewDialog>
#include <QPointer>
#include <QTextCodec>
#include <QWebFrame>

// KDE
#include <KDebug>
#include <KGlobalSettings>
#include <kio/accessmanager.h>

using namespace Mailody;

TotalView::TotalView( QWidget* parent )
        : QWidget( parent ), m_msg( 0 )
{
    QGridLayout* layout = new QGridLayout( this );

    m_headers = new MessageHeaderView( this );
    layout->addWidget( m_headers, 0, 0 );
    layout->setMargin( 0 );

    m_body = new MessageBodyView( this );
    layout->addWidget( m_body, 1, 0 );

    autoSaveAttachments();
}

TotalView::~TotalView()
{
}

void TotalView::setMsg( const Akonadi::Item& item, const Akonadi::Collection& collection )
{
    // disconnect to prevent multiple connections!
    if ( m_msg )
        delete m_msg;

    m_msg = new MessageData( this, "message" );

    connect( m_msg, SIGNAL( messageData( const MessageData* ) ),
             m_body, SLOT( slotShowMsg( const MessageData* ) ) );
    connect( m_msg, SIGNAL( possibleTagsAvailable( bool ) ),
             SIGNAL( possibleTagsAvailable( bool ) ) );

    bool ok = m_msg->setMsg( item, collection );
    kDebug() << "Loaded: " << ok;
    if ( !ok )
        return;

    m_nepomukWidget->setMsg( m_msg );
    m_headers->setMsg( m_msg );
    m_body->setMsg( m_msg );
}

void TotalView::updateNepomukWidget()
{
    m_nepomukWidget->setMsg( m_msg );
}

void TotalView::clearView()
{
    m_headers->clearView();
    m_body->clearView();
}

void TotalView::save( const QString& filename )
{
    QFile file( filename );
    QTextStream stream( &file );
    stream.setCodec( QTextCodec::codecForName( "UTF-8" ) );
    file.open( QIODevice::WriteOnly );
    stream << m_msg->raw().trimmed();
    file.close();
}

void TotalView::print()
{
    QString msgheadersdata;
    msgheadersdata += "<table border=1 cellspacing=0 "
                      "cellpadding=0 width=100%>";
    msgheadersdata += "<tr><td colspan=2 bgcolor=\"#dddddd\"><b><h2>"
                      + msg()->subject() + "<h2></b></td>";
    msgheadersdata += "</tr>";
    msgheadersdata += "<tr><td align=right><b>" + i18n( "From:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->sender_full( true )
                      + "</td>";
    msgheadersdata += "</tr>";
    msgheadersdata += "<tr><td align=right><b>" + i18n( "To:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->to() + "</td>";
    msgheadersdata += "</tr>";
    if ( !msg()->cc().isEmpty() ) {
        msgheadersdata += "<tr><td align=right><b>" + i18n( "Cc:" )
                          + "</b></td>";
        msgheadersdata += "    <td width=\"100%\">" + msg()->cc() + "</td>";
        msgheadersdata += "</tr>";
    }
    msgheadersdata += "<tr><td align=right><b>" + i18n( "Date:" ) + "</b></td>";
    msgheadersdata += "    <td width=\"100%\">" + msg()->vDate() + "</td>";
    msgheadersdata += "</tr></table>";

    m_printPart = new QWebPage( this );
    KIO::AccessManager* accessManager = new KIO::AccessManager( this );
    accessManager->setExternalContentAllowed( body()->externalImage() );
    m_printPart->setNetworkAccessManager( accessManager );
    m_printPart->settings()->setFontFamily( QWebSettings::StandardFont,
                                            m_body->fixedfont() ? KGlobalSettings::fixedFont().family() : KGlobalSettings::generalFont().family() );

    m_printPart->mainFrame()->setHtml( msgheadersdata  + "<br><br>" + msg()->body() );
    connect( m_printPart, SIGNAL( loadFinished( bool ) ), this, SLOT( slotPrintPreview() ) );
}

void TotalView::slotPrintPreview()
{
    QPointer<QPrintPreviewDialog> dlg = new QPrintPreviewDialog( m_printPart->view() );
    connect( dlg, SIGNAL( paintRequested( QPrinter * ) ),
             m_printPart->mainFrame(), SLOT( print( QPrinter * ) ) );
    dlg->exec();
    delete dlg;
    delete m_printPart;
}

#include "messageview.moc"
