/*
    Copyright (C) 2007 Tobias Koenig <tokoe@kde.org>
    Copyright (C) 2009 Omat Holding B.V. <info@omat.nl>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "recentproxymodel.h"
#include "recentproxymodelattributes.h"

#include <kabc/addressee.h>
#include <kdebug.h>
#include <kicon.h>
#include <klocale.h>
#include <akonadi/item.h>
#include <akonadi/itemfetchjob.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/attributefactory.h>

using namespace Akonadi;

class RecentProxyModel::Private
{
public:
};

RecentProxyModel::RecentProxyModel( QObject *parent )
        : QSortFilterProxyModel( parent ),
        d( new Private() )
{
}

RecentProxyModel::~RecentProxyModel()
{
    delete d;
}

int RecentProxyModel::columnCount( const QModelIndex& ) const
{
    return 4;
}

QVariant RecentProxyModel::data( const QModelIndex &index, int role ) const
{
    if ( role == ItemModel::IdRole )
        return QSortFilterProxyModel::data( index, role );

    if ( !index.isValid() )
        return QVariant();

    if ( index.row() >= rowCount() )
        return QVariant();

    Akonadi::ItemModel* model = static_cast<Akonadi::ItemModel*>( sourceModel() );

    const Item item = model->itemForIndex( mapToSource( index ) );

    if ( !item.hasPayload<KABC::Addressee>() ) {
        kDebug() << "No payload";
        return QVariant();
    }


    const KABC::Addressee addr = item.payload<KABC::Addressee>();

    if ( role == Qt::DisplayRole ) {
        switch ( index.column() ) {
        case 0:
            return addr.preferredEmail();
            break;
        case 1:
            if ( !addr.formattedName().isEmpty() )
                return addr.formattedName();
            else
                return addr.assembledName();
            break;
        case 2:
            if ( item.hasAttribute<AmountAttribute>() )
                return item.attribute<AmountAttribute>()->amount();
            else
                return 0;
            break;
        case 3:
            if ( item.hasAttribute<DateAttribute>() )
                return item.attribute<DateAttribute>()->date();
            else
                return( i18n( "Never" ) );
            break;
        default:
            break;
        }
    }

    return QVariant();
}

QVariant RecentProxyModel::headerData( int section, Qt::Orientation orientation, int role ) const
{
    if ( role != Qt::DisplayRole )
        return QVariant();

    if ( orientation != Qt::Horizontal )
        return QVariant();

    switch ( section ) {
    case 0:
        return KABC::Addressee::emailLabel();
        break;
    case 1:
        return i18nc( "@title:column, name of a person", "Name" );
        break;
    case 2:
        return i18nc( "@title:column, number of times you mailed a person", "Amount" );
        break;
    case 3:
        return i18nc( "@title:column, date of the last time you mailed that person", "Last" );
        break;
    default:
        break;
    }

    return QVariant();
}
