/* This file is part of the KDE project
   Copyright (C) 2005 Bram Schoenmakers <bramschoenmakers@kde.nl>
   Copyright (C) 2006 Tom Albers <tomalbers@kde.nl>
   Copyright (C) 2006-2007 Omat Holding B.V. <info@omat.nl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#ifndef TOOLTIP_H
#define TOOLTIP_H

#include <QMap>

#include <kpassivepopup.h>

class QMouseEvent;
class QContextMenuEvent;
class QLabel;

namespace Mailody
{

/**
 * @class ToolTip
 * This frame is shown as a tooltip, when there is new mail
 * @author Bram Schoenmakers <bramschoenmakers@kde.nl>
 */
class ToolTip : public KPassivePopup
{
    Q_OBJECT
public:
    /**
     * Constructor
     * @param parent The parent widget
     * @param name The name
     */
    explicit ToolTip( QWidget *parent = 0, const char *name = 0 );

    /**
     * Destruction
     */
    ~ToolTip();

    /**
     * Adds this mb to the blocklist. It will not popup for 10 seconds.
     * @param mb String to add to blocklist.
     */
    void addToBlockList( const QString& mb );

public slots:
    /**
     * Sets the tooltip to display a text
     * @param mb if mb is on the blocklist, don't show the text.
     * @param text the text to show.
     */
    void setText( const QString &mb, const QString &text );

    /**
     * Display a simple message
     * @param point The point to show it
     * @param text The text to show
     */
    void setText( const QPoint& point, const QString &text );

protected:
    /** Reimplemented */
    virtual void mouseReleaseEvent( QMouseEvent* );
    /** Reimplemented */
    virtual void contextMenuEvent( QContextMenuEvent* );

// @cond private
private:
    QLabel                  *mText;
    QTimer*                  m_clearTimer;
    QMap<QString, bool>      m_blockList;

private slots:
    void clearBlockList();
// @endcond

signals:
    /** emitted when left click is received */
    void leftClicked( const QString& );

    /** emitted when right click is received */
    void rightClicked( const QString& );
};

}

#endif
